@@ -1173,8 +1173,9 @@ class Annotations():
             duration[stage] = hypno.count(stage) * self.epoch_length / 60
 
         slp_onset = sorted(first.values(), key=lambda x: x[1]['start'])[0]
-        wake_up = next((len(epochs) - i, j) for i, j in enumerate(epochs[::-1]) if \
-                       j['stage'] in ['NREM1', 'NREM2', 'NREM3', 'REM'])
+        wake_up = next((len(epochs) - i, j) for i, j in enumerate(
+                epochs[::-1]) if j['stage'] in ['NREM1', 'NREM2', 'NREM3',
+                                                'REM'])
 
         total_dark_time = (lights_on - lights_out) / 60
         slp_period_time = (wake_up[1]['start'] - slp_onset[1]['start']) / 60
@@ -1224,7 +1225,7 @@ class Annotations():
                                duration['Artefact'],
                                ])
 
-        return slp_onset_lat # for testing
+        return slp_onset_lat, waso, total_slp_time # for testing
 
     def export_event_data(self, filename, summary, events, cycles=None,
                           fsplit=None):
