@@ -7,7 +7,7 @@ from numpy import (absolute, arange, argmax, argmin, asarray, concatenate, cos,
                    square, std, vstack, where, zeros)
 from scipy.ndimage.filters import gaussian_filter
 from scipy.signal import (argrelmax, butter, cheby2, filtfilt, fftconvolve, 
-                          hilbert, periodogram, tukey)
+                          hilbert, periodogram, remez, tukey)
 try:
     from PyQt5.QtCore import Qt
     from PyQt5.QtWidgets import QProgressDialog
@@ -41,9 +41,11 @@ class DetectSpindle:
         self.min_interval = 0
         self.det_thresh_hi = 0
         self.power_peaks = 'interval'
+        self.frequency = frequency
 
         if method == 'Ferrarelli2007':
-            self.frequency = (10, 16)
+            if self.frequency is None:
+                self.frequency = (10, 16)
             self.det_cheby2 = {'order': 6,
                                'freq': self.frequency,
                                }
@@ -55,7 +57,8 @@ class DetectSpindle:
             self.smooth = {'dur': None}
 
         elif method == 'Nir2011':
-            self.frequency = (10, 16)
+            if self.frequency is None:
+                self.frequency = (10, 16)
             self.det_butter = {'order': 2,
                                'freq': self.frequency,
                                }
@@ -68,7 +71,8 @@ class DetectSpindle:
             self.smooth = {'dur': .04}  # is in fact sigma
 
         elif method == 'Wamsley2012':
-            self.frequency = (12, 15)
+            if self.frequency is None:
+                self.frequency = (12, 15)
             self.det_wavelet = {'f0': mean(self.frequency),
                                 'sd': .5,
                                 'dur': 2,
@@ -80,7 +84,8 @@ class DetectSpindle:
             self.smooth = {'dur': .1}
 
         elif method == 'UCSD':
-            self.frequency = (10, 16)
+            if self.frequency is None:
+                self.frequency = (10, 16)
             self.det_wavelet = {'freqs': arange(self.frequency[0],
                                                 self.frequency[1] + .5, .5),
                                 'dur': 1,
@@ -102,9 +107,9 @@ class DetectSpindle:
             self.smooth = {'dur': None}
 
         elif method == 'Moelle2011':
-            self.frequency = (9, 15)
-            self.det_butter = {'order': 3,
-                               'freq': self.frequency,
+            if self.frequency is None:
+                self.frequency = (12, 15)
+            self.det_luebeck = {'freq': self.frequency,
                                }
             self.duration = (0.5, 3)
             self.det_wavelet = {'sd': None}
@@ -114,13 +119,14 @@ class DetectSpindle:
             self.smooth = {'dur': .2}
 
         elif method == 'Concordia':
-            self.frequency = (10, 16)
+            if self.frequency is None:
+                self.frequency = (10, 16)
             self.det_butter = {'order': 3,
                                'freq': self.frequency,
                                }
             self.duration = (0.5, 3)
             self.det_wavelet = {'sd': None}
-            self.det_thresh_lo = 1.5
+            self.det_thresh_lo = 3
             self.det_thresh_hi = 10
             self.sel_thresh = 1
             self.moving_rms = {'dur': .2}
@@ -174,9 +180,6 @@ class DetectSpindle:
         all_spindles = []
         i = 0
         for i, chan in enumerate(data.axis['chan'][0]):
-            
-            if parent is not None:
-                progress.setValue(i)
                 
             lg.info('Detecting spindles on chan %s', chan)
             time = hstack(data.axis['time'])
@@ -222,6 +225,11 @@ class DetectSpindle:
                 sp.update({'chan': chan})
 
             all_spindles.extend(sp_in_chan)
+            
+            if parent is not None:
+                progress.setValue(i)
+                if progress.wasCanceled():
+                    return
             # end of loop over chan
 
         spindle.events = sorted(all_spindles, key=lambda x: x['start'])
@@ -230,7 +238,7 @@ class DetectSpindle:
             spindle.events = merge_close(spindle.events, self.min_interval)
 
         if parent is not None:
-            progress.setValue(i + 1)
+            progress.setValue(i + 1)            
         
         return spindle
 
@@ -337,8 +345,8 @@ def detect_Moelle2011(dat_orig, s_freq, time, opts):
     s_freq : float
         sampling frequency
     opts : instance of 'DetectSpindle'
-        'det_butter' : dict
-            parameters for 'butter',
+        'det_luebeck' : dict
+            parameters for 'luebeck',
         'moving_rms' : dict
             parameters for 'moving_rms'
         'smooth' : dict
@@ -369,7 +377,7 @@ def detect_Moelle2011(dat_orig, s_freq, time, opts):
     ----------
     Moelle, M. et al. Sleep 34, 1411-21 (2011).
     """
-    dat_det = transform_signal(dat_orig, s_freq, 'butter', opts.det_butter)
+    dat_det = transform_signal(dat_orig, s_freq, 'luebeck', opts.det_luebeck)
     dat_det = transform_signal(dat_det, s_freq, 'moving_rms', opts.moving_rms)
     dat_det = transform_signal(dat_det, s_freq, 'moving_avg', opts.smooth)
 
@@ -806,6 +814,17 @@ def transform_signal(dat, s_freq, method, method_opt=None):
         b, a = butter(N, Wn, btype='lowpass')
         dat = filtfilt(b, a, dat)
     
+    if 'luebeck' == method:
+        N = int(s_freq * 2.36)
+        Fp1, Fp2 = method_opt['freq']
+        Fs1, Fs2 = Fp1 - 2, Fp2 + 2
+        nyquist = s_freq / 2
+        dens = 20
+        
+        bpass = remez(N, [0, Fs1, Fp1, Fp2, Fs2, nyquist], [0, 1, 0], 
+                      grid_density=dens, fs=s_freq)
+        dat = filtfilt(bpass, 1, dat)
+    
     if 'morlet' == method:
         f0 = method_opt['f0']
         sd = method_opt['sd']
