@@ -31,8 +31,8 @@ from PyQt5.QtWidgets import (QAction,
                              )
 
 from .. import ChanTime
-from ..trans import montage, filter_
-from .settings import Config, FormFloat, FormInt, FormBool
+from ..trans import montage, filter_, _select_channels
+from .settings import Config
 from .utils import (convert_name_to_color,
                     ICON,
                     LINE_COLOR,
@@ -40,6 +40,10 @@ from .utils import (convert_name_to_color,
                     Path,
                     RectMarker,
                     TextItem_with_BG,
+                    FormFloat,
+                    FormInt,
+                    FormBool,
+                    export_graphics,
                     )
 
 
@@ -285,6 +289,11 @@ class Traces(QGraphicsView):
         act.triggered.connect(self.display_annotations)
         actions['cross_chan_mrk'] = act
 
+        # Misc
+        act = QAction('Export to svg...', self)
+        act.triggered.connect(partial(export_graphics, MAIN=self.parent))
+        actions['export_svg'] = act
+
         self.action = actions
 
     def read_data(self):
@@ -345,7 +354,7 @@ class Traces(QGraphicsView):
         self.idx_markers = []
         self.idx_annot = []
         self.idx_annot_labels = []
-        
+
         self.add_chan_labels()
         self.add_time_labels()
         self.add_traces()
@@ -424,7 +433,7 @@ class Traces(QGraphicsView):
         self.chan = []
         self.chan_pos = []
         self.chan_scale = []
-        
+
         row = 0
         for one_grp in self.parent.channels.groups:
             for one_chan in one_grp['chan_to_plot']:
@@ -448,7 +457,7 @@ class Traces(QGraphicsView):
                 self.chan.append(chan_name)
                 self.chan_scale.append(one_grp['scale'])
                 self.chan_pos.append(chan_pos)
-                
+
     def display_grid(self):
         """Display grid on x-axis and y-axis."""
         window_start = self.parent.value('window_start')
@@ -1085,7 +1094,7 @@ def _create_data_to_plot(data, chan_groups):
 
         if one_grp['hp'] is not None:
             data1 = filter_(data1, low_cut=one_grp['hp'])
-        
+
         if one_grp['lp'] is not None:
             data1 = filter_(data1, high_cut=one_grp['lp'])
 
@@ -1103,37 +1112,6 @@ def _create_data_to_plot(data, chan_groups):
     return output
 
 
-def _select_channels(data, channels):
-    """Select channels.
-
-    Parameters
-    ----------
-    data : instance of ChanTime
-        data with all the channels
-    channels : list
-        channels of interest
-
-    Returns
-    -------
-    instance of ChanTime
-        data with only channels of interest
-
-    Notes
-    -----
-    This function does the same as wonambi.trans.select, but it's much
-    faster. wonambi.trans.Select needs to flexible for any data type, here
-    we assume that we have one trial, and that channel is the first dimension.
-
-    """
-    output = data._copy()
-    chan_list = list(data.axis['chan'][0])
-    idx_chan = [chan_list.index(i_chan) for i_chan in channels]
-    output.data[0] = data.data[0][idx_chan, :]
-    output.axis['chan'][0] = asarray(channels)
-
-    return output
-
-
 def _convert_timestr_to_seconds(time_str, rec_start):
     """Convert input from user about time string to an absolute time for
     the recordings.
